#include "Corrector.h"

Corrector::Corrector(PClip _child, PClip *clps, int nbclp, int mode, int th, IScriptEnvironment *env) :
GenericVideoFilter(_child)
{
   mvclips = clps;

   nClips = nbclp;
   nWidth = vi.width;
   nHeight = vi.height;

   if ( mode == 1)
      Process = ProcessBest;
   else
      Process = ProcessMean;

   nThreshold = th;
}

Corrector::~Corrector()
{
}

void ProcessMean(Uint8 *pDst, const Uint8 *pSrc, const Uint8 **pSrcs,
                 int nDstPitch, int nSrcPitch, int *pPitches, int width, int height, int nThreshold, int nClips)
{
   int nSum;
   int nCount;
   for ( int y = 0; y < height; y++ )
   {
      for ( int x = 0; x < width; x++ )
      {
         nCount = nSum = 0;
         for ( int i = 0; i < nClips; i++ )
         {
            if ( abs(pSrcs[i][x] - pSrc[x]) < nThreshold )
            {
               nSum += pSrcs[i][x];
               nCount++;
            }
         }
         if ( nCount )
            pDst[x] = (nSum + (nCount >> 1)) / nCount;
         else
            pDst[x] = pSrc[x];
      }
      for ( int i = 0; i < nClips; i++ )
         pSrcs[i] += pPitches[i];
      pSrc += nSrcPitch;
      pDst += nDstPitch;
   }
}

void ProcessBest(Uint8 *pDst, const Uint8 *pSrc, const Uint8 **pSrcs,
                 int nDstPitch, int nSrcPitch, int *pPitches, int width, int height, int nThreshold, int nClips)
{
   int nMin;
   int nVal;
   for ( int y = 0; y < height; y++ )
   {
      for ( int x = 0; x < width; x++ )
      {
         nMin = nThreshold;
         nVal = -1;
         for ( int i = 0; i < nClips; i++ )
         {
            if ( abs(pSrcs[i][x] - pSrc[x]) < nMin )
            {
               nVal = pSrcs[i][x];
               nMin = abs(pSrcs[i][x] - pSrc[x]);
            }
         }
         if ( nVal == -1 )
            pDst[x] = pSrc[x];
         else
            pDst[x] = nVal;
      }
      for ( int i = 0; i < nClips; i++ )
         pSrcs[i] += pPitches[i];
      pSrc += nSrcPitch;
      pDst += nDstPitch;
   }
}


PVideoFrame __stdcall Corrector::GetFrame(int n, IScriptEnvironment* env)
{
   PVideoFrame src = child->GetFrame(n, env);
   PVideoFrame dst = env->NewVideoFrame(vi);
   env->MakeWritable(&dst);

   const Uint8 **pSrcs = new const Uint8*[nClips];
   int *pPitches = new int[nClips];
   for ( int i = 0; i < nClips; i++ )
   {
      pSrcs[i] = YRPLAN(mvclips[i]->GetFrame(n, env));
      pPitches[i] = YPITCH(mvclips[i]->GetFrame(n, env));
   }

   Process(YWPLAN(dst), YRPLAN(src), pSrcs, YPITCH(dst), YPITCH(src), pPitches, nWidth, nHeight, nThreshold, nClips);

   for ( int i = 0; i < nClips; i++ )
   {
      pSrcs[i] = URPLAN(mvclips[i]->GetFrame(n, env));
      pPitches[i] = UPITCH(mvclips[i]->GetFrame(n, env));
   }

   Process(UWPLAN(dst), URPLAN(src), pSrcs, UPITCH(dst), UPITCH(src), pPitches, nWidth / 2, nHeight / 2, nThreshold, nClips);

   for ( int i = 0; i < nClips; i++ )
   {
      pSrcs[i] = VRPLAN(mvclips[i]->GetFrame(n, env));
      pPitches[i] = VPITCH(mvclips[i]->GetFrame(n, env));
   }

   Process(VWPLAN(dst), VRPLAN(src), pSrcs, VPITCH(dst), VPITCH(src), pPitches, nWidth / 2, nHeight / 2, nThreshold, nClips);

   delete[] pSrcs;
   delete[] pPitches;

   return dst;
}