// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#ifndef __FPOBLOCKS__
#define __FPOBLOCKS__

#include "portability.h"
#include "FakeBlockData.h"
#include "commonfunctions.h"

class FakePlaneOfBlocks {
    
	int nWidth;
	int nHeight;
	int nBlkX;
	int nBlkY;
	int nBlkSize;
	int nBlkCount;
	int nPel;
	int nLogPel;
	int nScale;
	int nLogScale;

	FakeBlockData **blocks;

	void SetNeighbours();

public :

	FakePlaneOfBlocks(int w, int h, int size, int lv, int pel);
	~FakePlaneOfBlocks();

	void Update(const int *array, const unsigned char *pRef, int nPitch);
	bool IsSceneChange(int nTh1, int nTh2) const;

	MOTION_INLINE bool IsInFrame(int i) const
	{
		return (( i >= 0 ) && ( i < nBlkCount ));
	}
	
	MOTION_INLINE const FakeBlockData& operator[](const int i) const {
		return *(blocks[i]);
	}

	MOTION_INLINE int GetBlockCount() const { return nBlkCount; }
	MOTION_INLINE int GetReducedWidth() const { return nBlkX; }
	MOTION_INLINE int GetReducedHeight() const { return nBlkY; }
	MOTION_INLINE int GetWidth() const { return nWidth; }
	MOTION_INLINE int GetHeight() const { return nHeight; }
	MOTION_INLINE int GetScaleLevel() const { return nLogScale; }
	MOTION_INLINE int GetEffectiveScale() const { return nScale; }
	MOTION_INLINE int GetBlockSize() const { return nBlkSize; }
	MOTION_INLINE int GetPel() const { return nPel; }
   MOTION_INLINE const FakeBlockData& GetBlock(int i) const { return *(blocks[i]); }
};

#endif
