// General classe for motion based filters

// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#include "GenericMotionFilter.h"
#include "stdio.h"

GenericMotionFilter::GenericMotionFilter(PClip _child, const MotionParameters &params, const VideoInfo &vi2, IScriptEnvironment* env) :
GenericVideoFilter(_child), MotionParameters(params)
{
	if (!vi.IsYV12())
		env->ThrowError("GenericMotionFilter: Needs YV12 data");

   // we fetch the handle on the analyze filter
   MVAnalyse *pAnalyseFilter = reinterpret_cast<MVAnalyse *>(vi2.nchannels);

   // 'magic' key, just to check : 
   if ( pAnalyseFilter->GetMagicKey() != MOTION_MAGIC_KEY )
      env->ThrowError("GenericMotionFilter : unvalid vectors stream");

   // we fetch the analysis' parameters
   nBlkSize = pAnalyseFilter->GetBlkSize();
   nPel = pAnalyseFilter->GetPel();
   isBackward = pAnalyseFilter->GetIsBackward();
   nLvCount = pAnalyseFilter->GetLevelCount();
   nDeltaFrame = pAnalyseFilter->GetDeltaFrame();

   nVPadding = nBlkSize;
   nHPadding = nBlkSize;

	nWidth = vi.width;
	nHeight = vi.height;

	nBlkX = nWidth / nBlkSize;
	nBlkY = nHeight / nBlkSize;
   nBlkCount = nBlkX * nBlkY;

	mmx = mmx && ( env->GetCPUFlags() & CPUF_MMX );
	isse = isse && ( env->GetCPUFlags() & CPUF_INTEGER_SSE );

	if ( nWidth & 7 ) env->ThrowError("GenericMotionFilter: Width has to be dividable by 8");
	if ( nHeight & 7 ) env->ThrowError("GenericMotionFilter: Height has to be dividable by 8");

   nThSCD2 = (nWidth * nHeight * nThSCD2 / (256 * nBlkSize * nBlkSize));
}

GenericMotionFilter::~GenericMotionFilter()
{
}

void GenericMotionFilter::CreateFGOP(FakeGroupOfPlanes **fgop)
{
	*fgop = new FakeGroupOfPlanes(nWidth, nHeight, nBlkSize, nLvCount, nPel);
}

bool GenericMotionFilter::IsSceneChange(FakeGroupOfPlanes *fgop)
{
	return fgop->IsSceneChange(nThSCD1, nThSCD2);
}

bool GenericMotionFilter::IsUsable(FakeGroupOfPlanes *fgop)
{
	return (!IsSceneChange(fgop)) && fgop->GetValidity();
}

void GenericMotionFilter::SetVectorsNeed(IScriptEnvironment *env, const PClip &pclip, 
                                         bool srcluma, bool refluma, bool var,
                                         bool compy, bool compu, bool compv)
{
   int nFlags = 0;

   nFlags |= srcluma ? MOTION_CALC_SRC_LUMA        : 0;
   nFlags |= refluma ? MOTION_CALC_REF_LUMA        : 0;
   nFlags |= var     ? MOTION_CALC_VAR             : 0;
   nFlags |= compy   ? MOTION_COMPENSATE_LUMA      : 0;
   nFlags |= compu   ? MOTION_COMPENSATE_CHROMA_U  : 0;
   nFlags |= compv   ? MOTION_COMPENSATE_CHROMA_V  : 0;

   MVAnalyse *pAnalyseFilter = reinterpret_cast<MVAnalyse *>(pclip->GetVideoInfo().nchannels);
   pAnalyseFilter->SetFlags(nFlags);
}

void GenericMotionFilter::GetVectorStream(int n, IScriptEnvironment* env, const PClip &pclip, FakeGroupOfPlanes *fgop )
{
	return fgop->Update(reinterpret_cast<const int*>(pclip->GetFrame(n,env)->GetReadPtr()));
}



const FakeBlockData &GenericMotionFilter::GetBlock(FakeGroupOfPlanes *fgop, int lv, int blk)
{
	return fgop->GetPlane(lv)[blk];
}