// Set of tools based on Motion Compensation

// See legal notice in Copying.txt for more information

// Avisynth: www.avisynth.org

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#define WIN32_LEAN_AND_MEAN

#include "AviSynth.h"
#include "portability.h"
#include "MVCore.h"

#if 0
#include "MVTracking.h"
#include "MVCreateFrame.h"
#include "MVBlur.h"
#include "MVDenoise.h"
#include "MVConvertFPS.h"
#include "EncDenoise.h"
#endif

#include "MVMask.h"
#include "MVDenoise2.h"
#include "MVShow.h"
#include "MVSpeed.h"
#include "MVCompensate.h"
#include "MVChangeCompensate.h"
#include "MVSCDetection.h"

#include "MVBidouille.h"

#include "MVAnalyse.h"

#include "TestFilter.h"
#include "Padding.h"
#include "QDeQuant.h"
#include "Deblock.h"
#include "YUVSource.h"
#include "Corrector.h"

#include "MVDepan.h" // added by Fizick


AVSValue __cdecl Create_Padding(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new Padding(
      args[0].AsClip(),
      args[1].AsInt(8),
		args[2].AsInt(8),
		env);
}

AVSValue __cdecl Create_MVChangeCompensate(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new MVChangeCompensate(
		args[0].AsClip(),
		args[1].AsClip(),
		env);
}

AVSValue __cdecl Create_QDenoise(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new QDeQuant(
		args[0].AsClip(),
		args[1].AsInt(25),
      args[2].AsInt(0),
      args[3].AsInt(6),
		args[4].AsBool(true),
      args[5].AsBool(true),
		env);
}

AVSValue __cdecl Create_Deblock(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new Deblock(
		args[0].AsClip(),
      args[1].AsInt(25),
      args[2].AsInt(0),
      args[3].AsInt(0),
		args[4].AsBool(true),
      args[5].AsBool(true),
    	env);
}

AVSValue __cdecl Create_MVDenoise2(AVSValue args, void* user_data, IScriptEnvironment* env)
{
   int nbmv = args[1].ArraySize();
   if (nbmv <= 0) env->ThrowError("MVDenoise2 needs at least one vector's stream to work");
   int thT = 10;
	int sadT = 200;
	int thMV = 30;
   PClip *mvs = new PClip[nbmv];

   int nMode = 0;
   nMode |= args[2].AsBool(true) ? YPLANE : 0;
   nMode |= args[3].AsBool(true) ? UPLANE : 0;
   nMode |= args[4].AsBool(true) ? VPLANE : 0;

   for ( int i=0; i < nbmv; ++i)
      mvs[i] = args[1][i].AsClip();

	return new MVDenoise2(
		args[0].AsClip(),
      mvs,
      nbmv,
      nMode,
		args[5].AsInt(thT),
		args[6].AsInt(sadT),
		args[7].AsInt(thMV),
		MotionParameters(args, 8),
		env);
}

AVSValue __cdecl Create_Corrector(AVSValue args, void* user_data, IScriptEnvironment* env)
{
   int nbclp = args[1].ArraySize();
   if (nbclp <= 0) env->ThrowError("Corrector needs at least two clips to work");
   PClip *clps = new PClip[nbclp];

   for ( int i=0; i < nbclp; ++i)
      clps[i] = args[1][i].AsClip();

	return new Corrector(
		args[0].AsClip(),
      clps,
      nbclp,
      args[2].AsInt(0),
      args[3].AsInt(10),
		env);
}

AVSValue __cdecl Create_MVShow(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int sc = 1;
	int sil = 0;
	int tol = 20000;
	return new MVShow(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsInt(sc),
		args[3].AsInt(sil),
		args[4].AsInt(tol),
		args[5].AsBool(false),
		MotionParameters(args, 6),
		env);
}

AVSValue __cdecl Create_MVSpeed(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new MVSpeed(
		args[0].AsClip(),
		args[1].AsClip(),
      args[2].AsBool(true),
      args[3].AsBool(true),
      args[4].AsBool(true),
      args[5].AsBool(true),
      args[6].AsBool(true),
      args[7].AsBool(true),
		MotionParameters(args, 8),
		env);
}

AVSValue __cdecl Create_MVCompensate(AVSValue args, void* user_data, IScriptEnvironment* env)
{
   bool inClip = false;
   bool inLoop = false;
   if ( args[3].AsInt(0) == 2 ) inLoop = true;
   else if ( args[3].AsInt(0) == 1 ) inClip = true;
	return new MVCompensate(
		args[0].AsClip(),
		args[1].AsClip(),
      args[2].AsBool(true), 
      inClip,
      inLoop,
		MotionParameters(args, 4),
		env);
}

AVSValue __cdecl Create_MVSCDetection(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new MVSCDetection(
		args[0].AsClip(),
		args[1].AsClip(),
      args[2].AsInt(255),
		MotionParameters(args, 3),
		env);
}

AVSValue __cdecl Create_MVAnalyse(AVSValue args, void* user_data, IScriptEnvironment* env)
{
   return new MVAnalyse(
		args[0].AsClip(),
		args[1].AsInt(8),             // block size
		args[2].AsInt(1),             // pel
		args[3].AsInt(0),             // level
		args[4].AsInt(LOGARITHMIC),   // search type
		args[5].AsInt(2),             // search parameter
		args[6].AsBool(false),        // is backward
		args[7].AsInt(0),             // lambda
		args[8].AsInt(1),             // delta frame
		args[9].AsBool(true),         // mmx
		args[10].AsBool(true),        // isse
      args[11].AsInt(MVCore::mvCore.GetNextIdx()),   // index of the frames for this analysis
		env);
}

AVSValue __cdecl Create_MVBidouille(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new MVBidouille(
		args[0].AsClip(),
		args[1].AsClip(),
      args[2].AsClip(),
      args[3].AsInt(MVCore::mvCore.GetNextIdx()),
		MotionParameters(args,4),
		env);
}

AVSValue __cdecl Create_MVMask(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int ml = 100; double gamma = 1;
   int Ysc = 0;
	return new MVMask(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsInt(ml),
		args[3].AsFloat(gamma),
		args[4].AsBool(false),
      args[5].AsInt(Ysc),
		MotionParameters(args, 6),
		env);
}

#if 0
AVSValue __cdecl Create_MVTracking(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int x = 100;
	int y = 100;
	int w = 100;
	int h = 100;
	return new MVTracking(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsInt(x),
		args[3].AsInt(y),
		args[4].AsInt(w),
		args[5].AsInt(h),
		MotionParameters(args,6),
		env);
}

AVSValue __cdecl Create_MVBlur(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int nb = 10;
	double rl = 0.5;
	int fbw = 2;
	char wf[] = "uniform";
	return new MVBlur(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsClip(),
		args[3].AsInt(nb),
		args[4].AsFloat(rl),
		args[5].AsInt(fbw),
		args[6].AsString(wf),
		MotionParameters(args,7),
		env);
}

AVSValue __cdecl Create_MVDenoise(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int thT = 10;
	int thMC = 10;
	int sadT = 200;
	int thMV = 30;
	return new MVDenoise(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsClip(),
		args[3].AsInt(thT),
		args[4].AsInt(thMC),
		args[5].AsInt(sadT),
		args[6].AsInt(thMV),
		MotionParameters(args,7),
		env);
}

AVSValue __cdecl Create_EncDenoise(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new EncDenoise(
		args[0].AsClip(),
      args[1].AsClip(),
      args[2].AsBool(true),
      args[3].AsInt(25),
      args[4].AsInt(0),
      args[5].AsInt(0),
      args[6].AsInt(0),
      args[7].AsInt(2),
      MotionParameters(args, 8),
		env);
}

AVSValue __cdecl Create_MVConvertFPS(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int nb = 5;
	double rl = 0.2;
	int num = -1;
	int den = 1;
	double fps = -1;
	int fbw = 4;
	int sbw = 0;
	int rbw = 0;
	int bvt = 35000;
	char wf[] = "uniform";
	return new MVConvertFPS(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsClip(),
		args[3].AsInt(nb),
		args[4].AsFloat(rl),
		args[5].AsInt(fbw),
		args[6].AsInt(sbw),
		args[7].AsInt(rbw),
		args[8].AsInt(bvt),
		args[9].AsInt(num),
		args[10].AsInt(den),
		args[11].AsFloat(fps),
		args[12].AsString(wf),
		MotionParameters(args,13),
		env);
}

AVSValue __cdecl Create_MVInterpolate(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	int nb = 5;
	double bl = 0.4;
	double el = 0.6;
	int fbw = 4;
	int sbw = 0;
	int rbw = 0;
	int bvt = 35000;
	char wf[] = "uniform";
	return new MVCreateFrame(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsClip(),
		args[3].AsInt(nb),
		args[4].AsFloat(bl),
		args[5].AsFloat(el),
		args[6].AsInt(fbw),
		args[7].AsInt(sbw),
		args[8].AsInt(rbw),
		args[9].AsInt(bvt),
		args[10].AsString(wf),
		MotionParameters(args,11),
		env);
}
#endif

AVSValue __cdecl Create_TestFilter(AVSValue args, void* user_data, IScriptEnvironment* env)
{
	return new TestFilter(
		args[0].AsClip(),
		env);
}

AVSValue __cdecl Create_YUVSource(AVSValue args, void *user_data, IScriptEnvironment *env)
{
   return new YUVSource(
      args[0].AsString(""),
      args[1].AsInt(720),
      args[2].AsInt(576),
      env);
}

AVSValue __cdecl Create_MVDepan(AVSValue args, void* user_data, IScriptEnvironment* env) // Added by Fizick
{
	return new MVDepan(
		args[0].AsClip(),
		args[1].AsClip(),
		args[2].AsBool(true), // zoom
		args[3].AsBool(true), // rot
		(float)args[4].AsFloat(1), // pixaspect
		(float)args[5].AsFloat(15), // error
		args[6].AsBool(false), // info
		MotionParameters(args, 7),
		env);
}


extern "C" __declspec(dllexport) const char* __stdcall
AvisynthPluginInit2(IScriptEnvironment* env) {

//	env->AddFunction("MVDenoise", "c[mvbw]c[mvfw]c[thT]i[thMC]i[sadt]i[thMV]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVDenoise, 0);
	env->AddFunction("MVDenoise", "cc+[Y]b[U]b[V]b[thT]i[thSAD]i[thMV]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVDenoise2, 0);
	env->AddFunction("MVShow", "c[vectors]c[scale]i[sil]i[tol]i[showsad]b[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVShow, 0);
	env->AddFunction("MVSpeed", "c[vectors]c[srcluma]b[refluma]b[var]b[compy]b[compu]b[compv]b[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVSpeed, 0);
	env->AddFunction("MVAnalyse", "c[blksize]i[pel]i[level]i[search]i[searchparam]i[isb]b[lambda]i[delta]i[mmx]b[isse]b[idx]i", Create_MVAnalyse, 0);
	env->AddFunction("MVMask", "c[vectors]c[ml]i[gamma]f[showsad]b[Ysc]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVMask, 0);
//	env->AddFunction("MVBlur", "c[mvbw]c[mvfw]c[nb]i[rl]f[fbw]i[wf]s[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVBlur, 0);
   env->AddFunction("MVBidouille", "c[mvbw]c[mvfw]c[idx]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVBidouille, 0);
//	env->AddFunction("MVConvertFPS", "c[mvbw]c[mvfw]c[nb]i[rl]f[fbw]i[sbw]i[rbw]i[bvt]i[num]i[den]i[fps]f[wf]s[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVConvertFPS, 0);
//	env->AddFunction("MVInterpolate", "c[mvbw]c[mvfw]c[nb]i[bl]f[el]f[fbw]i[sbw]i[rbw]i[bvt]i[wf]s[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVInterpolate, 0);
//	env->AddFunction("MVTracking", "cc[x]i[y]i[w]i[h]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVTracking, 0);
	env->AddFunction("MVCompensate", "c[vectors]c[scbehavior]b[mode]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVCompensate, 0);
   env->AddFunction("MVChangeCompensate", "cc", Create_MVChangeCompensate, 0);
   env->AddFunction("MVSCDetection", "c[vectors]c[Yth]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVSCDetection, 0);
	env->AddFunction("TestFilter","c", Create_TestFilter, 0);
	env->AddFunction("Padding","c[hPad]i[vPad]i", Create_Padding, 0);
   env->AddFunction("YUVSource","[file]s[width]i[height]i", Create_YUVSource, 0);
   env->AddFunction("Corrector", "cc+[mode]i[th]i", Create_Corrector, 0);
// env->AddFunction("QDeQuant", "c[quant]i[level]i[deadzone]i[mmx]b[isse]b", Create_QDenoise, 0);
   env->AddFunction("Deblock", "c[quant]i[aOffset]i[bOffset]i[mmx]b[isse]b", Create_Deblock, 0);
// env->AddFunction("EncDenoise", "c[vectors]c[scbehavior]b[quant]i[aOffset]i[bOffset]i[level]i[deadzone]i[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_EncDenoise, 0);
	env->AddFunction("MVDepan", "c[vectors]c[zoom]b[rot]b[pixaspect]f[error]f[info]b[thSCD1]i[thSCD2]i[mmx]b[isse]b", Create_MVDepan, 0);
	return("MVTools : set of tools based on a motion estimation engine");
}
