// Make a motion compensate temporal denoiser

// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#ifndef __MV_ANALYSE__
#define __MV_ANALYSE__

#include "portability.h"
#include "avisynth.h"
#include "GroupOfPlanes.h"
#include "MotionParameters.h"
#include "commonfunctions.h"
#include <stdio.h>
#include "MVCore.h"

class MVAnalyse : public GenericVideoFilter {
protected:
	/*! \brief Width of the frame */
	int nWidth;
	
	/*! \brief Height of the frame */
	int nHeight;

	/*! \brief Frames of blocks for which motion vectors will be computed */
	GroupOfPlanes *vectorFields;

   /*! \brief size of a block, in pixel */
   int nBlkSize;

   /*! \brief pixel refinement of the motion estimation */
   int nPel;

   /*! \brief number of level for the hierarchal search */
   int nLvCount;

   /*! \brief mmx optimisations enabled */
   bool mmx;

   /*! \brief isse optimisations enabled */
	bool isse;

   /*! \brief motion vecteur cost factor */
   int nLambda;

   /*! \brief difference between the index of the reference and the index of the current frame */
   int nDeltaFrame;

   /*! \brief search type chosen for refinement in the EPZ */
   SearchType searchType;

   /*! \brief additionnal parameter for this search */
	int nSearchParam;

   /*! \brief direction of the search ( forward / backward ) */
	bool isBackward;

   /*! \brief diverse flags to set up the search */
   int nFlags;

   /*! \brief MVFrames idx */
   int nIdx;

   int nMagicKey;

public :


	MVAnalyse(PClip _child, int size, int pel, int lv, int st, int stp, bool isb,
             int lambda, int df, bool _mmx, bool _isse, int _idx, IScriptEnvironment* env);
	~MVAnalyse();

   MOTION_INLINE void SetFlags(int _nFlags) { nFlags |= _nFlags; }
   MOTION_INLINE int GetFlags() { return nFlags; }
   MOTION_INLINE int GetBlkSize() { return nBlkSize; }
   MOTION_INLINE int GetPel() { return nPel; }
   MOTION_INLINE int GetLevelCount() { return nLvCount; }
   MOTION_INLINE int GetFramesIdx() { return nIdx; }
   MOTION_INLINE bool GetIsBackward() { return isBackward; }
   MOTION_INLINE int GetMagicKey() { return nMagicKey; }
   MOTION_INLINE int GetDeltaFrame() { return nDeltaFrame; }

	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

};

#endif