// Make a motion compensate temporal denoiser

// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#include "MVChangeCompensate.h"
#include "portability.h"
#include "MVAnalyse.h"

MVChangeCompensate::MVChangeCompensate(PClip _child, PClip _video, IScriptEnvironment* env) :
GenericVideoFilter(_child), newcomp(_video)
{
   MVAnalyse *pAnalyseFilter = reinterpret_cast<MVAnalyse *>(vi.nchannels);

   nLevelCount = pAnalyseFilter->GetLevelCount();
   nWidth = newcomp->GetVideoInfo().width;
   nHeight = newcomp->GetVideoInfo().height;
}

MVChangeCompensate::~MVChangeCompensate()
{
}

PVideoFrame __stdcall MVChangeCompensate::GetFrame(int n, IScriptEnvironment* env)
{
	PVideoFrame	src	= child->GetFrame(n, env);
   PVideoFrame newframe = newcomp->GetFrame(n, env);
   PVideoFrame dst = env->NewVideoFrame(vi);

   const int *pSrc = reinterpret_cast<const int *>(src->GetReadPtr());
   const int *s = pSrc + 2;
   int nMvSize = 2;
   for ( int i = 0; i < nLevelCount; i++ )
   {
      nMvSize += s[0];
      s += s[0];
   }
   nMvSize++;
  
   nMvSize *= 4;

   env->BitBlt(dst->GetWritePtr(), nMvSize, src->GetReadPtr(), nMvSize, nMvSize, 1);
   env->BitBlt(dst->GetWritePtr() + nMvSize, nWidth, YRPLAN(newframe), YPITCH(newframe), nWidth, nHeight);
   env->BitBlt(dst->GetWritePtr() + nMvSize + nWidth * nHeight, nWidth / 2, URPLAN(newframe), UPITCH(newframe), nWidth / 2, nHeight / 2);
   env->BitBlt(dst->GetWritePtr() + nMvSize + nWidth * nHeight * 5 / 4, nWidth / 2, VRPLAN(newframe), VPITCH(newframe), nWidth / 2, nHeight / 2);

   return dst;
}