// Create an overlay mask with the motion vectors

// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#include "MVMask.h"

#include <math.h>
#include <memory.h>

MVMask::MVMask(PClip _child, PClip mvs, int ml, double gm, bool ss, int Ysc, const MotionParameters &params, IScriptEnvironment* env) :
GenericMotionFilter(_child, params, mvs->GetVideoInfo(), env), mvclip(mvs)
{
	if ( ml < 0 )
        ml = 0;
	nLengthMax = pow((double)ml,gm);

	nGamma = gm / 2;
	if ( nGamma < 0 )
        nGamma = 0.5;

	showSad = ss;

   nSceneChangeValue = (Ysc < 0) ? 0 : ((Ysc > 255) ? 255 : Ysc);

	smallMask = new unsigned char[nBlkX * nBlkY];

  	destinations = new unsigned char *[nBlkSize * nBlkSize];

	for ( int i = 0; i < nBlkSize * nBlkSize; i++ )
		destinations[i] = new unsigned char[nBlkX * nBlkY];

	upsizer = new Upsizer(nBlkX, nBlkY, nBlkSize, nBlkSize, nBlkSize / 2,
                         nBlkSize / 2, 0, 0, new BilinearResampling());

	CreateFGOP(&fgop);
}


MVMask::~MVMask()
{
	delete smallMask;
	for ( int i = 0; i < nBlkSize * nBlkSize; i++ )
		delete[] destinations[i];
	delete[] destinations;
	delete upsizer;
}

void MVMask::Reorganize(unsigned char *d, int dp)
{
	int bhdp = nBlkSize * dp;
	for ( int i = 0; i < nBlkSize; i++ )
	{
		int jdp = 0;
		for ( int j = 0; j < nBlkSize; j++ )
		{
			const unsigned char *dest = destinations[i + j * nBlkSize];
			int kbhdp = 0;
			int krw = 0;
			for ( int k = 0; k < nBlkY; k++ )
			{
				int mbw = 0;
				for ( int m = 0; m < nBlkX; m++ )
				{
					d[mbw + i + kbhdp + jdp] = dest[krw + m];
					mbw += nBlkSize;
				}
				kbhdp += bhdp;
				krw += nBlkX;
			}
			jdp += dp;
		}
	}
}

unsigned char MVMask::Length(VECTOR v, unsigned char pel)
{
	double norme = (v.x * v.x + v.y * v.y) / ( pel * pel);

	double l = 255 * pow(norme,nGamma) / nLengthMax;

	return (unsigned char)((l > 255) ? 255 : l) ;
}

unsigned char MVMask::SAD(unsigned int s)
{
	double l = 255 * pow(s, nGamma) / nLengthMax;
	return (unsigned char)((l > 255) ? 255 : l);
}

PVideoFrame __stdcall MVMask::GetFrame(int n, IScriptEnvironment* env)
{
	PVideoFrame	dst = env->NewVideoFrame(vi);

	GetVectorStream(n, env, mvclip, fgop);

	if ( IsUsable(fgop) )
	{
		for ( int j = 0; j < nBlkCount; j++)
			smallMask[j] = (showSad) ? SAD(fgop->GetPlane(0)[j].GetSAD()) : Length(fgop->GetPlane(0)[j].GetMV(), nPel);
		upsizer->Resize(smallMask, destinations, nBlkX, nBlkX, true);
		Reorganize(YWPLAN(dst), YPITCH(dst));
	}
	else {
		memset(YWPLAN(dst), nSceneChangeValue, YPITCH(dst) * nHeight);
	}

	memset(UWPLAN(dst), 0, UPITCH(dst) * nHeight / 2);
	memset(VWPLAN(dst), 0, VPITCH(dst) * nHeight / 2);

	return dst;
}