// Make a motion compensate temporal denoiser

// See legal notice in Copying.txt for more information

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .

#include "MVSCDetection.h"
#include "CopyCode.h"

MVSCDetection::MVSCDetection(PClip _child, PClip vectors, int Ysc, const MotionParameters &params, IScriptEnvironment* env) :
GenericMotionFilter(_child, params, vectors->GetVideoInfo(), env), mvclip(vectors)
{
	CreateFGOP(&fgop);
   sceneChangeValue = (Ysc < 0) ? 0 : ((Ysc > 255) ? 255 : Ysc);
}

MVSCDetection::~MVSCDetection()
{
}

PVideoFrame __stdcall MVSCDetection::GetFrame(int n, IScriptEnvironment* env)
{
   PVideoFrame dst = env->NewVideoFrame(vi);

   GetVectorStream(n, env, mvclip, fgop);

	if ( IsUsable(fgop) )
	{
      memset(YWPLAN(dst), 0, YPITCH(dst) * nHeight);
      memset(UWPLAN(dst), 0, UPITCH(dst) * nHeight / 2);
      memset(VWPLAN(dst), 0, VPITCH(dst) * nHeight / 2);
	}
   else {
      memset(YWPLAN(dst), sceneChangeValue, YPITCH(dst) * nHeight);
      memset(UWPLAN(dst), sceneChangeValue, UPITCH(dst) * nHeight / 2);
      memset(VWPLAN(dst), sceneChangeValue, VPITCH(dst) * nHeight / 2);
   }

	return dst;
}